@@ -10,10 +10,11 @@
     missing_docs
 )]
 #![allow(dead_code)] // TEMP
+#![allow(clippy::too_many_arguments, clippy::type_complexity)]
 
-//! Hanabi -- a particle system plugin for the Bevy game engine.
+//! Hanabi -- a GPU particle system plugin for the Bevy game engine.
 //!
-//! This library provides a particle system for the Bevy game engine.
+//! This library provides a GPU-based particle system for the Bevy game engine.
 //!
 //! # Example
 //!
@@ -28,7 +29,7 @@
 //!     .run();
 //! ```
 //!
-//! Create an EffectAsset describing a visual effect, then add an
+//! Create an [`EffectAsset`] describing a visual effect, then add an
 //! instance of that effect to an entity:
 //!
 //! ```
@@ -101,6 +102,9 @@ pub use plugin::HanabiPlugin;
 pub use render::EffectCacheId;
 pub use spawn::{Spawner, Value};
 
+#[cfg(not(any(feature = "2d", feature = "3d")))]
+compile_error!("Enable either the '2d' or '3d' feature.");
+
 /// Extension trait to write a floating point scalar or vector constant in a format
 /// matching the WGSL grammar.
 ///
@@ -123,50 +127,47 @@ pub trait ToWgslString {
 impl ToWgslString for f32 {
     fn to_wgsl_string(&self) -> String {
         let s = format!("{:.6}", self);
-        s.trim_end_matches("0").to_string()
+        s.trim_end_matches('0').to_string()
     }
 }
 
 impl ToWgslString for f64 {
     fn to_wgsl_string(&self) -> String {
         let s = format!("{:.15}", self);
-        s.trim_end_matches("0").to_string()
+        s.trim_end_matches('0').to_string()
     }
 }
 
 impl ToWgslString for Vec2 {
     fn to_wgsl_string(&self) -> String {
-        let s = format!(
+        format!(
             "vec2<f32>({0}, {1})",
             self.x.to_wgsl_string(),
             self.y.to_wgsl_string()
-        );
-        s.to_string()
+        )
     }
 }
 
 impl ToWgslString for Vec3 {
     fn to_wgsl_string(&self) -> String {
-        let s = format!(
+        format!(
             "vec3<f32>({0}, {1}, {2})",
             self.x.to_wgsl_string(),
             self.y.to_wgsl_string(),
             self.z.to_wgsl_string()
-        );
-        s.to_string()
+        )
     }
 }
 
 impl ToWgslString for Vec4 {
     fn to_wgsl_string(&self) -> String {
-        let s = format!(
+        format!(
             "vec4<f32>({0}, {1}, {2}, {3})",
             self.x.to_wgsl_string(),
             self.y.to_wgsl_string(),
             self.z.to_wgsl_string(),
             self.w.to_wgsl_string()
-        );
-        s.to_string()
+        )
     }
 }
 
@@ -221,7 +222,7 @@ impl ParticleEffect {
     /// adding modifiers to the effect.
     pub fn spawner(&mut self, spawner: &Spawner) -> &mut Spawner {
         if self.spawner.is_none() {
-            self.spawner = Some(spawner.clone());
+            self.spawner = Some(*spawner);
         }
         self.spawner.as_mut().unwrap()
     }
@@ -249,7 +250,7 @@ mod tests {
         assert_eq!(s, "1.5");
         let s = 0.5_f32.to_wgsl_string();
         assert_eq!(s, "0.5");
-        let s = 0.123456789_f32.to_wgsl_string();
+        let s = 0.123_456_78_f32.to_wgsl_string();
         assert_eq!(s, "0.123457"); // 6 digits
     }
 
@@ -263,7 +264,7 @@ mod tests {
         assert_eq!(s, "1.5");
         let s = 0.5_f64.to_wgsl_string();
         assert_eq!(s, "0.5");
-        let s = 0.1234567890123456789_f64.to_wgsl_string();
+        let s = 0.123_456_789_012_345_67_f64.to_wgsl_string();
         assert_eq!(s, "0.123456789012346"); // 15 digits
     }
 
