@@ -1,9 +1,13 @@
+#[cfg(feature = "2d")]
+use bevy::core_pipeline::Transparent2d;
+#[cfg(feature = "3d")]
+use bevy::core_pipeline::Transparent3d;
 use bevy::{
-    core_pipeline::Transparent3d,
     prelude::*,
     render::{
         render_graph::RenderGraph, render_phase::DrawFunctions,
-        render_resource::SpecializedPipelines, renderer::RenderDevice, RenderApp, RenderStage,
+        render_resource::SpecializedRenderPipelines, renderer::RenderDevice, RenderApp,
+        RenderStage,
     },
 };
 
@@ -12,14 +16,14 @@ use crate::{
     render::{
         extract_effect_events, extract_effects, prepare_effects, queue_effects, ComputeCache,
         DrawEffects, EffectAssetEvents, EffectBindGroups, EffectSystems, EffectsMeta,
-        ExtractedEffects, ImageBindGroups, ParticleUpdateNode, ParticlesRenderPipeline,
-        ParticlesUpdatePipeline, PipelineRegistry, SimParams, PARTICLES_RENDER_SHADER_HANDLE,
+        ExtractedEffects, ParticleUpdateNode, ParticlesRenderPipeline, ParticlesUpdatePipeline,
+        PipelineRegistry, SimParams, PARTICLES_RENDER_SHADER_HANDLE,
         PARTICLES_UPDATE_SHADER_HANDLE,
     },
     spawn::{self, Random},
 };
 
-pub mod draw_3d_graph {
+pub mod draw_graph {
     pub mod node {
         /// Label for the particle update compute node.
         pub const PARTICLE_UPDATE_PASS: &str = "particle_update_pass";
@@ -59,12 +63,11 @@ impl Plugin for HanabiPlugin {
         let render_app = app.sub_app_mut(RenderApp);
         render_app
             .insert_resource(effects_meta)
-            .init_resource::<ImageBindGroups>()
             .init_resource::<EffectBindGroups>()
             .init_resource::<ParticlesUpdatePipeline>()
             .init_resource::<ComputeCache<ParticlesUpdatePipeline>>()
             .init_resource::<ParticlesRenderPipeline>()
-            .init_resource::<SpecializedPipelines<ParticlesRenderPipeline>>()
+            .init_resource::<SpecializedRenderPipelines<ParticlesRenderPipeline>>()
             .init_resource::<ExtractedEffects>()
             .init_resource::<EffectAssetEvents>()
             .init_resource::<SimParams>()
@@ -86,8 +89,19 @@ impl Plugin for HanabiPlugin {
             );
 
         // Register the draw function for drawing the particles. This will be called during
-        // the main 3D pass, at the Transparent3d phase, after the opaque objects have been
+        // the main 2D/3D pass, at the Transparent2d/3d phase, after the opaque objects have been
         // rendered (or, rather, commands for those have been recorded).
+        #[cfg(feature = "2d")]
+        {
+            let draw_particles = DrawEffects::new(&mut render_app.world);
+            render_app
+                .world
+                .get_resource::<DrawFunctions<Transparent2d>>()
+                .unwrap()
+                .write()
+                .add(draw_particles);
+        }
+        #[cfg(feature = "3d")]
         {
             let draw_particles = DrawEffects::new(&mut render_app.world);
             render_app
@@ -98,30 +112,62 @@ impl Plugin for HanabiPlugin {
                 .add(draw_particles);
         }
 
-        // Register the update node before the 3D main pass, where the particles are drawn.
+        // Register the update node before the 2D/3D main pass, where the particles are drawn.
         // This ensures the update compute pipelines for all the active particle effects are
-        // executed before the 3D main pass starts, which consumes the result of the updated
+        // executed before the 2D/3D main pass starts, which consumes the result of the updated
         // particles to render them.
-        let update_node = ParticleUpdateNode::new(&mut render_app.world);
+        #[cfg(feature = "2d")]
+        use bevy::core_pipeline::draw_2d_graph;
+        #[cfg(feature = "3d")]
+        use bevy::core_pipeline::draw_3d_graph;
+
+        #[cfg(feature = "2d")]
+        let update_node_2d = ParticleUpdateNode::new(&mut render_app.world);
+
+        #[cfg(feature = "3d")]
+        let update_node_3d = ParticleUpdateNode::new(&mut render_app.world);
+
         let mut graph = render_app.world.get_resource_mut::<RenderGraph>().unwrap();
-        let draw_3d_graph = graph
-            .get_sub_graph_mut(bevy::core_pipeline::draw_3d_graph::NAME)
-            .unwrap();
-        draw_3d_graph.add_node(draw_3d_graph::node::PARTICLE_UPDATE_PASS, update_node);
-        draw_3d_graph
-            .add_node_edge(
-                draw_3d_graph::node::PARTICLE_UPDATE_PASS,
-                bevy::core_pipeline::draw_3d_graph::node::MAIN_PASS,
-            )
-            .unwrap();
-        draw_3d_graph
-            .add_slot_edge(
-                draw_3d_graph.input_node().unwrap().id,
-                bevy::core_pipeline::draw_3d_graph::input::VIEW_ENTITY,
-                draw_3d_graph::node::PARTICLE_UPDATE_PASS,
-                ParticleUpdateNode::IN_VIEW,
-            )
-            .unwrap();
+
+        #[cfg(feature = "2d")]
+        {
+            let draw_graph = graph.get_sub_graph_mut(draw_2d_graph::NAME).unwrap();
+            draw_graph.add_node(draw_graph::node::PARTICLE_UPDATE_PASS, update_node_2d);
+            draw_graph
+                .add_node_edge(
+                    draw_graph::node::PARTICLE_UPDATE_PASS,
+                    draw_2d_graph::node::MAIN_PASS,
+                )
+                .unwrap();
+            draw_graph
+                .add_slot_edge(
+                    draw_graph.input_node().unwrap().id,
+                    draw_2d_graph::input::VIEW_ENTITY,
+                    draw_graph::node::PARTICLE_UPDATE_PASS,
+                    ParticleUpdateNode::IN_VIEW,
+                )
+                .unwrap();
+        }
+
+        #[cfg(feature = "3d")]
+        {
+            let draw_graph = graph.get_sub_graph_mut(draw_3d_graph::NAME).unwrap();
+            draw_graph.add_node(draw_graph::node::PARTICLE_UPDATE_PASS, update_node_3d);
+            draw_graph
+                .add_node_edge(
+                    draw_graph::node::PARTICLE_UPDATE_PASS,
+                    draw_3d_graph::node::MAIN_PASS,
+                )
+                .unwrap();
+            draw_graph
+                .add_slot_edge(
+                    draw_graph.input_node().unwrap().id,
+                    draw_3d_graph::input::VIEW_ENTITY,
+                    draw_graph::node::PARTICLE_UPDATE_PASS,
+                    ParticleUpdateNode::IN_VIEW,
+                )
+                .unwrap();
+        }
     }
 }
 
