@@ -3,13 +3,12 @@
 use bevy::{
     asset::{AssetEvent, Assets, Handle, HandleId, HandleUntyped},
     core::{cast_slice, FloatOrd, Pod, Time, Zeroable},
-    core_pipeline::Transparent3d,
     ecs::{
         prelude::*,
         system::{lifetimeless::*, SystemState},
     },
     log::trace,
-    math::{const_vec3, Mat4, Vec2, Vec3, Vec4, Vec4Swizzles},
+    math::{const_vec3, Mat4, Rect, Vec2, Vec3, Vec4, Vec4Swizzles},
     reflect::TypeUuid,
     render::{
         color::Color,
@@ -22,7 +21,6 @@ use bevy::{
         view::{ComputedVisibility, ExtractedView, ViewUniform, ViewUniformOffset, ViewUniforms},
         RenderWorld,
     },
-    sprite::Rect,
     transform::components::GlobalTransform,
     utils::{HashMap, HashSet},
 };
@@ -33,6 +31,11 @@ use rand::Rng;
 use std::sync::atomic::{AtomicU64, Ordering as AtomicOrdering};
 use std::{borrow::Cow, cmp::Ordering, num::NonZeroU64, ops::Range};
 
+#[cfg(feature = "2d")]
+use bevy::core_pipeline::Transparent2d;
+#[cfg(feature = "3d")]
+use bevy::core_pipeline::Transparent3d;
+
 use crate::{
     asset::EffectAsset,
     modifiers::{ForceFieldParam, FFNUM},
@@ -54,10 +57,10 @@ pub const PARTICLES_UPDATE_SHADER_HANDLE: HandleUntyped =
 pub const PARTICLES_RENDER_SHADER_HANDLE: HandleUntyped =
     HandleUntyped::weak_from_u64(Shader::TYPE_UUID, 2763343953151597145);
 
-const PARTICLES_UPDATE_SHADER_TEMPLATE: &'static str = include_str!("particles_update.wgsl");
-const PARTICLES_RENDER_SHADER_TEMPLATE: &'static str = include_str!("particles_render.wgsl");
+const PARTICLES_UPDATE_SHADER_TEMPLATE: &str = include_str!("particles_update.wgsl");
+const PARTICLES_RENDER_SHADER_TEMPLATE: &str = include_str!("particles_render.wgsl");
 
-const DEFAULT_POSITION_CODE: &'static str = r##"
+const DEFAULT_POSITION_CODE: &str = r##"
     ret.pos = vec3<f32>(0., 0., 0.);
     var dir = rand3() * 2. - 1.;
     dir = normalize(dir);
@@ -65,12 +68,12 @@ const DEFAULT_POSITION_CODE: &'static str = r##"
     ret.vel = dir * speed;
 "##;
 
-const DEFAULT_FORCE_FIELD_CODE: &'static str = r##"
+const DEFAULT_FORCE_FIELD_CODE: &str = r##"
     vVel = vVel + (spawner.accel * sim_params.dt);
     vPos = vPos + vVel * sim_params.dt;
 "##;
 
-const FORCE_FIELD_CODE: &'static str = include_str!("force_field_code.wgsl");
+const FORCE_FIELD_CODE: &str = include_str!("force_field_code.wgsl");
 
 /// Labels for the Hanabi systems.
 #[derive(Debug, Hash, PartialEq, Eq, Clone, SystemLabel)]
@@ -93,7 +96,7 @@ trait ShaderCode {
 
 impl ShaderCode for Gradient<Vec2> {
     fn to_shader_code(&self) -> String {
-        if self.keys().len() == 0 {
+        if self.keys().is_empty() {
             return String::new();
         }
         let mut s: String = self
@@ -135,7 +138,7 @@ impl ShaderCode for Gradient<Vec2> {
 
 impl ShaderCode for Gradient<Vec4> {
     fn to_shader_code(&self) -> String {
-        if self.keys().len() == 0 {
+        if self.keys().is_empty() {
             return String::new();
         }
         let mut s: String = self
@@ -221,15 +224,15 @@ pub struct ForceFieldStd430 {
     pub conform_to_sphere: f32,
 }
 
-impl Into<ForceFieldStd430> for ForceFieldParam {
-    fn into(self) -> ForceFieldStd430 {
+impl From<ForceFieldParam> for ForceFieldStd430 {
+    fn from(param: ForceFieldParam) -> Self {
         ForceFieldStd430 {
-            position_or_direction: self.position,
-            max_radius: self.max_radius,
-            min_radius: self.min_radius,
-            mass: self.mass,
-            force_exponent: self.force_exponent,
-            conform_to_sphere: if self.conform_to_sphere { 1.0 } else { 0.0 },
+            position_or_direction: param.position,
+            max_radius: param.max_radius,
+            min_radius: param.min_radius,
+            mass: param.mass,
+            force_exponent: param.force_exponent,
+            conform_to_sphere: if param.conform_to_sphere { 1.0 } else { 0.0 },
         }
     }
 }
@@ -447,22 +450,13 @@ impl FromWorld for ParticlesRenderPipeline {
     }
 }
 
-#[derive(Clone, Hash, PartialEq, Eq)]
+#[derive(Default, Clone, Hash, PartialEq, Eq)]
 pub struct ParticleUpdatePipelineKey {
     /// Code for the position initialization of newly emitted particles.
     position_code: String,
     force_field_code: String,
 }
 
-impl Default for ParticleUpdatePipelineKey {
-    fn default() -> Self {
-        ParticleUpdatePipelineKey {
-            position_code: Default::default(),
-            force_field_code: Default::default(),
-        }
-    }
-}
-
 impl SpecializedComputePipeline for ParticlesUpdatePipeline {
     type Key = ParticleUpdatePipelineKey;
 
@@ -479,17 +473,22 @@ impl SpecializedComputePipeline for ParticlesUpdatePipeline {
             source: ShaderSource::Wgsl(Cow::Owned(source)),
         });
 
-        let pipeline = render_device.create_compute_pipeline(&ComputePipelineDescriptor {
+        render_device.create_compute_pipeline(&RawComputePipelineDescriptor {
             label: Some("particles_update_compute_pipeline"),
             layout: Some(&self.pipeline_layout),
             module: &shader_module,
             entry_point: "main",
-        });
-
-        pipeline
+        })
     }
 }
 
+#[cfg(all(feature = "2d", feature = "3d"))]
+#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
+pub enum PipelineMode {
+    Camera2d,
+    Camera3d,
+}
+
 #[derive(Clone, Hash, PartialEq, Eq)]
 pub struct ParticleRenderPipelineKey {
     /// Render shader, with template applied, but not preprocessed yet.
@@ -498,6 +497,10 @@ pub struct ParticleRenderPipelineKey {
     /// Define a texture sampled to modulate the particle color.
     /// This key requires the presence of UV coordinates on the particle vertices.
     particle_texture: Option<Handle<Image>>,
+    /// For dual-mode configurations only, the actual mode of the current render
+    /// pipeline. Otherwise the mode is implicitly determined by the active feature.
+    #[cfg(all(feature = "2d", feature = "3d"))]
+    pipeline_mode: PipelineMode,
 }
 
 impl Default for ParticleRenderPipelineKey {
@@ -505,11 +508,13 @@ impl Default for ParticleRenderPipelineKey {
         ParticleRenderPipelineKey {
             shader: PARTICLES_RENDER_SHADER_HANDLE.typed::<Shader>(),
             particle_texture: None,
+            #[cfg(all(feature = "2d", feature = "3d"))]
+            pipeline_mode: PipelineMode::Camera3d,
         }
     }
 }
 
-impl SpecializedPipeline for ParticlesRenderPipeline {
+impl SpecializedRenderPipeline for ParticlesRenderPipeline {
     type Key = ParticleRenderPipelineKey;
 
     fn specialize(&self, key: Self::Key) -> RenderPipelineDescriptor {
@@ -570,6 +575,33 @@ impl SpecializedPipeline for ParticlesRenderPipeline {
             // vertex_buffer_layout.array_stride += 8;
         }
 
+        #[cfg(all(feature = "2d", feature = "3d"))]
+        let depth_stencil = match key.pipeline_mode {
+            // Bevy's Transparent2d render phase doesn't support a depth-stencil buffer.
+            PipelineMode::Camera2d => None,
+            PipelineMode::Camera3d => Some(DepthStencilState {
+                format: TextureFormat::Depth32Float,
+                depth_write_enabled: false,
+                // Bevy uses reverse-Z, so Greater really means closer
+                depth_compare: CompareFunction::Greater,
+                stencil: StencilState::default(),
+                bias: DepthBiasState::default(),
+            }),
+        };
+
+        #[cfg(all(feature = "2d", not(feature = "3d")))]
+        let depth_stencil: Option<DepthStencilState> = None;
+
+        #[cfg(all(feature = "3d", not(feature = "2d")))]
+        let depth_stencil = Some(DepthStencilState {
+            format: TextureFormat::Depth32Float,
+            depth_write_enabled: false,
+            // Bevy uses reverse-Z, so Greater really means closer
+            depth_compare: CompareFunction::Greater,
+            stencil: StencilState::default(),
+            bias: DepthBiasState::default(),
+        });
+
         RenderPipelineDescriptor {
             vertex: VertexState {
                 shader: key.shader.clone(),
@@ -597,14 +629,7 @@ impl SpecializedPipeline for ParticlesRenderPipeline {
                 topology: PrimitiveTopology::TriangleList,
                 strip_index_format: None,
             },
-            depth_stencil: Some(DepthStencilState {
-                format: TextureFormat::Depth32Float,
-                depth_write_enabled: false,
-                // Bevy uses reverse-Z, so Greater really means closer
-                depth_compare: CompareFunction::Greater,
-                stencil: StencilState::default(),
-                bias: DepthBiasState::default(),
-            }),
+            depth_stencil,
             multisample: MultisampleState {
                 count: 4, // TODO: Res<Msaa>.samples
                 mask: !0,
@@ -633,7 +658,7 @@ pub struct ExtractedEffect {
     force_field: [ForceFieldParam; FFNUM],
     /// Particles tint to modulate with the texture image.
     pub color: Color,
-    pub rect: Rect,
+    pub rect: Rect<f32>,
     // Texture to use for the sprites of the particles of this effect.
     //pub image: Handle<Image>,
     pub has_image: bool, // TODO -> use flags
@@ -716,16 +741,16 @@ pub(crate) fn extract_effects(
     mut shaders: ResMut<Assets<Shader>>,
     mut pipeline_registry: ResMut<PipelineRegistry>,
     mut rng: ResMut<Random>,
-    mut query: QuerySet<(
+    mut query: ParamSet<(
         // All existing ParticleEffect components
-        QueryState<(
+        Query<(
             Entity,
             &ComputedVisibility,
             &mut ParticleEffect, //TODO - Split EffectAsset::Spawner (desc) and ParticleEffect::SpawnerData (runtime data), and init the latter on component add without a need for the former
             &GlobalTransform,
         )>,
         // Newly added ParticleEffect components
-        QueryState<
+        Query<
             (Entity, &mut ParticleEffect),
             (
                 Added<ParticleEffect>,
@@ -751,7 +776,7 @@ pub(crate) fn extract_effects(
 
     // Collect added effects for later GPU data allocation
     extracted_effects.added_effects = query
-        .q1()
+        .p1()
         .iter()
         .map(|(entity, effect)| {
             let handle = effect.handle.clone_weak();
@@ -766,7 +791,7 @@ pub(crate) fn extract_effects(
         .collect();
 
     // Loop over all existing effects to update them
-    for (entity, computed_visibility, mut effect, transform) in query.q0().iter_mut() {
+    for (entity, computed_visibility, mut effect, transform) in query.p0().iter_mut() {
         // Check if visible
         if !computed_visibility.is_visible {
             continue;
@@ -845,10 +870,12 @@ pub(crate) fn extract_effects(
                     accel,
                     force_field,
                     rect: Rect {
-                        min: Vec2::ZERO,
-                        max: Vec2::new(0.2, 0.2), // effect
-                                                  //.custom_size
-                                                  //.unwrap_or_else(|| Vec2::new(size.width as f32, size.height as f32)),
+                        left: -0.1,
+                        top: -0.1,
+                        right: 0.1,
+                        bottom: 0.1, // effect
+                                     //.custom_size
+                                     //.unwrap_or_else(|| Vec2::new(size.width as f32, size.height as f32)),
                     },
                     has_image: asset.render_layout.particle_texture.is_some(),
                     image_handle_id: asset
@@ -1077,7 +1104,7 @@ pub(crate) fn prepare_effects(
         .effects
         .iter()
         .map(|(entity, extracted_effect)| {
-            let slice = effects_meta.entity_map.get(&entity).unwrap().clone();
+            let slice = effects_meta.entity_map.get(entity).unwrap().clone();
             (slice, extracted_effect)
         })
         .collect::<Vec<_>>();
@@ -1254,9 +1281,9 @@ pub(crate) fn prepare_effects(
             handle: asset.clone_weak(),
             layout_flags,
             image_handle_id,
-            shader: shader.clone(),
-            position_code: position_code.clone(),
-            force_field_code: force_field_code.clone(),
+            shader,
+            position_code,
+            force_field_code,
             compute_pipeline: None,
         },));
         num_emitted += 1;
@@ -1273,11 +1300,6 @@ pub(crate) fn prepare_effects(
         .write_buffer(&render_device, &render_queue);
 }
 
-#[derive(Default)]
-pub struct ImageBindGroups {
-    values: HashMap<Handle<Image>, BindGroup>,
-}
-
 #[derive(Default)]
 pub struct EffectBindGroups {
     /// Bind groups for each group index for compute shader.
@@ -1288,24 +1310,27 @@ pub struct EffectBindGroups {
     update_indirect_buffers: HashMap<u32, BindGroup>,
     /// Bind groups for each indirect buffer associated with each particle buffer (render stage).
     render_indirect_buffers: HashMap<u32, BindGroup>,
+    ///
+    images: HashMap<Handle<Image>, BindGroup>,
 }
 
 #[allow(clippy::too_many_arguments)]
 pub(crate) fn queue_effects(
-    draw_functions: Res<DrawFunctions<Transparent3d>>,
+    #[cfg(feature = "2d")] draw_functions_2d: Res<DrawFunctions<Transparent2d>>,
+    #[cfg(feature = "3d")] draw_functions_3d: Res<DrawFunctions<Transparent3d>>,
     render_device: Res<RenderDevice>,
     mut effects_meta: ResMut<EffectsMeta>,
     view_uniforms: Res<ViewUniforms>,
     update_pipeline: Res<ParticlesUpdatePipeline>,
     mut compute_cache: ResMut<ComputeCache<ParticlesUpdatePipeline>>,
     render_pipeline: Res<ParticlesRenderPipeline>,
-    mut specialized_render_pipelines: ResMut<SpecializedPipelines<ParticlesRenderPipeline>>,
-    mut render_pipeline_cache: ResMut<RenderPipelineCache>,
-    mut image_bind_groups: ResMut<ImageBindGroups>,
+    mut specialized_render_pipelines: ResMut<SpecializedRenderPipelines<ParticlesRenderPipeline>>,
+    mut render_pipeline_cache: ResMut<PipelineCache>,
     mut effect_bind_groups: ResMut<EffectBindGroups>,
     gpu_images: Res<RenderAssets<Image>>,
     mut effect_batches: Query<(Entity, &mut EffectBatch)>,
-    mut views: Query<&mut RenderPhase<Transparent3d>>,
+    #[cfg(feature = "2d")] mut views_2d: Query<&mut RenderPhase<Transparent2d>>,
+    #[cfg(feature = "3d")] mut views_3d: Query<&mut RenderPhase<Transparent3d>>,
     events: Res<EffectAssetEvents>,
 ) {
     trace!("queue_effects");
@@ -1314,8 +1339,8 @@ pub(crate) fn queue_effects(
     for event in &events.images {
         match event {
             AssetEvent::Created { .. } => None,
-            AssetEvent::Modified { handle } => image_bind_groups.values.remove(handle),
-            AssetEvent::Removed { handle } => image_bind_groups.values.remove(handle),
+            AssetEvent::Modified { handle } => effect_bind_groups.images.remove(handle),
+            AssetEvent::Removed { handle } => effect_bind_groups.images.remove(handle),
         };
     }
 
@@ -1476,110 +1501,192 @@ pub(crate) fn queue_effects(
         batch.compute_pipeline = Some(compute_pipeline.clone());
     }
 
-    // Loop over all cameras/views that need to render effects
-    let draw_effects_function = draw_functions.read().get_id::<DrawEffects>().unwrap();
-    for mut transparent_phase in views.iter_mut() {
-        trace!("Process new Transparent3d view");
-        // For each view, loop over all the effect batches to determine if the effect needs to be rendered
-        // for that view, and enqueue a view-dependent batch if so.
-        for (entity, batch) in effect_batches.iter() {
-            trace!(
-                "Process batch entity={:?} buffer_index={} spawner_base={} slice={:?}",
-                entity,
-                batch.buffer_index,
-                batch.spawner_base,
-                batch.slice
-            );
-            // Ensure the particle texture is available as a GPU resource and create a bind group for it
-            let particle_texture = if batch.layout_flags.contains(LayoutFlags::PARTICLE_TEXTURE) {
-                let image_handle = Handle::weak(batch.image_handle_id);
-                if image_bind_groups.values.get(&image_handle).is_none() {
-                    trace!(
-                        "Batch buffer #{} slice={:?} has missing GPU image bind group, creating...",
-                        batch.buffer_index,
-                        batch.slice
-                    );
-                    // If texture doesn't have a bind group yet from another instance of the same effect,
-                    // then try to create one now
-                    if let Some(gpu_image) = gpu_images.get(&image_handle) {
-                        let bind_group = render_device.create_bind_group(&BindGroupDescriptor {
-                            entries: &[
-                                BindGroupEntry {
-                                    binding: 0,
-                                    resource: BindingResource::TextureView(&gpu_image.texture_view),
-                                },
-                                BindGroupEntry {
-                                    binding: 1,
-                                    resource: BindingResource::Sampler(&gpu_image.sampler),
-                                },
-                            ],
-                            label: Some("particles_material_bind_group"),
-                            layout: &render_pipeline.material_layout,
-                        });
-                        image_bind_groups
-                            .values
-                            .insert(image_handle.clone(), bind_group);
-                        Some(image_handle)
+    // Loop over all 2D cameras/views that need to render effects
+    #[cfg(feature = "2d")]
+    {
+        let draw_effects_function_2d = draw_functions_2d.read().get_id::<DrawEffects>().unwrap();
+        for mut transparent_phase_2d in views_2d.iter_mut() {
+            trace!("Process new Transparent2d view");
+            // For each view, loop over all the effect batches to determine if the effect needs to be rendered
+            // for that view, and enqueue a view-dependent batch if so.
+            for (entity, batch) in effect_batches.iter() {
+                trace!(
+                    "Process batch entity={:?} buffer_index={} spawner_base={} slice={:?}",
+                    entity,
+                    batch.buffer_index,
+                    batch.spawner_base,
+                    batch.slice
+                );
+                // Ensure the particle texture is available as a GPU resource and create a bind group for it
+                let particle_texture = if batch.layout_flags.contains(LayoutFlags::PARTICLE_TEXTURE)
+                {
+                    let image_handle = Handle::weak(batch.image_handle_id);
+                    if effect_bind_groups.images.get(&image_handle).is_none() {
+                        trace!(
+                            "Batch buffer #{} slice={:?} has missing GPU image bind group, creating...",
+                            batch.buffer_index,
+                            batch.slice
+                        );
+                        // If texture doesn't have a bind group yet from another instance of the same effect,
+                        // then try to create one now
+                        if let Some(gpu_image) = gpu_images.get(&image_handle) {
+                            let bind_group =
+                                render_device.create_bind_group(&BindGroupDescriptor {
+                                    entries: &[
+                                        BindGroupEntry {
+                                            binding: 0,
+                                            resource: BindingResource::TextureView(
+                                                &gpu_image.texture_view,
+                                            ),
+                                        },
+                                        BindGroupEntry {
+                                            binding: 1,
+                                            resource: BindingResource::Sampler(&gpu_image.sampler),
+                                        },
+                                    ],
+                                    label: Some("particles_material_bind_group"),
+                                    layout: &render_pipeline.material_layout,
+                                });
+                            effect_bind_groups
+                                .images
+                                .insert(image_handle.clone(), bind_group);
+                            Some(image_handle)
+                        } else {
+                            // Texture is not ready; skip for now...
+                            trace!("GPU image not yet available; skipping batch for now.");
+                            None
+                        }
                     } else {
-                        // Texture is not ready; skip for now...
-                        trace!("GPU image not yet available; skipping batch for now.");
-                        None
+                        // Bind group already exists, meaning texture is ready
+                        Some(image_handle)
                     }
                 } else {
-                    // Bind group already exists, meaning texture is ready
-                    Some(image_handle)
-                }
-            } else {
-                // Batch doesn't use particle texture
-                None
-            };
+                    // Batch doesn't use particle texture
+                    None
+                };
 
-            // Specialize the render pipeline based on the effect batch
-            trace!(
-                "Specializing render pipeline: shader={:?} particle_texture={:?}",
-                batch.shader,
-                particle_texture
-            );
-            let render_pipeline_id = specialized_render_pipelines.specialize(
-                &mut render_pipeline_cache,
-                &render_pipeline,
-                ParticleRenderPipelineKey {
-                    particle_texture,
-                    shader: batch.shader.clone(),
-                },
-            );
-            trace!("Render pipeline specialized: id={:?}", render_pipeline_id);
+                // Specialize the render pipeline based on the effect batch
+                trace!(
+                    "Specializing render pipeline: shader={:?} particle_texture={:?}",
+                    batch.shader,
+                    particle_texture
+                );
+                let render_pipeline_id = specialized_render_pipelines.specialize(
+                    &mut render_pipeline_cache,
+                    &render_pipeline,
+                    ParticleRenderPipelineKey {
+                        particle_texture,
+                        shader: batch.shader.clone(),
+                        #[cfg(feature = "3d")]
+                        pipeline_mode: PipelineMode::Camera2d,
+                    },
+                );
+                trace!("Render pipeline specialized: id={:?}", render_pipeline_id);
 
-            // Add a draw pass for the effect batch
-            trace!("Add Transparent3d for batch on entity {:?}: buffer_index={} spawner_base={} slice={:?} handle={:?}", entity, batch.buffer_index, batch.spawner_base, batch.slice, batch.handle);
-            transparent_phase.add(Transparent3d {
-                draw_function: draw_effects_function,
-                pipeline: render_pipeline_id,
-                entity,
-                distance: 0.0, // TODO ??????
-            });
+                // Add a draw pass for the effect batch
+                trace!("Add Transparent for batch on entity {:?}: buffer_index={} spawner_base={} slice={:?} handle={:?}", entity, batch.buffer_index, batch.spawner_base, batch.slice, batch.handle);
+                transparent_phase_2d.add(Transparent2d {
+                    draw_function: draw_effects_function_2d,
+                    pipeline: render_pipeline_id,
+                    entity,
+                    sort_key: FloatOrd(0.0),
+                    batch_range: None,
+                });
+            }
         }
     }
-}
 
-/// Draw function for rendering all active effects for the current frame.
-///
-/// Effects are rendered in the [`Transparent3d`] phase of the main 3D pass.
-pub struct DrawEffects {
-    params: SystemState<(
-        SRes<EffectsMeta>,
-        SRes<ImageBindGroups>,
-        SRes<EffectBindGroups>,
-        SRes<RenderPipelineCache>,
-        SQuery<Read<ViewUniformOffset>>,
-        SQuery<Read<EffectBatch>>,
-    )>,
-}
+    // Loop over all 3D cameras/views that need to render effects
+    #[cfg(feature = "3d")]
+    {
+        let draw_effects_function_3d = draw_functions_3d.read().get_id::<DrawEffects>().unwrap();
+        for mut transparent_phase_3d in views_3d.iter_mut() {
+            trace!("Process new Transparent3d view");
+            // For each view, loop over all the effect batches to determine if the effect needs to be rendered
+            // for that view, and enqueue a view-dependent batch if so.
+            for (entity, batch) in effect_batches.iter() {
+                trace!(
+                    "Process batch entity={:?} buffer_index={} spawner_base={} slice={:?}",
+                    entity,
+                    batch.buffer_index,
+                    batch.spawner_base,
+                    batch.slice
+                );
+                // Ensure the particle texture is available as a GPU resource and create a bind group for it
+                let particle_texture = if batch.layout_flags.contains(LayoutFlags::PARTICLE_TEXTURE)
+                {
+                    let image_handle = Handle::weak(batch.image_handle_id);
+                    if effect_bind_groups.images.get(&image_handle).is_none() {
+                        trace!(
+                            "Batch buffer #{} slice={:?} has missing GPU image bind group, creating...",
+                            batch.buffer_index,
+                            batch.slice
+                        );
+                        // If texture doesn't have a bind group yet from another instance of the same effect,
+                        // then try to create one now
+                        if let Some(gpu_image) = gpu_images.get(&image_handle) {
+                            let bind_group =
+                                render_device.create_bind_group(&BindGroupDescriptor {
+                                    entries: &[
+                                        BindGroupEntry {
+                                            binding: 0,
+                                            resource: BindingResource::TextureView(
+                                                &gpu_image.texture_view,
+                                            ),
+                                        },
+                                        BindGroupEntry {
+                                            binding: 1,
+                                            resource: BindingResource::Sampler(&gpu_image.sampler),
+                                        },
+                                    ],
+                                    label: Some("particles_material_bind_group"),
+                                    layout: &render_pipeline.material_layout,
+                                });
+                            effect_bind_groups
+                                .images
+                                .insert(image_handle.clone(), bind_group);
+                            Some(image_handle)
+                        } else {
+                            // Texture is not ready; skip for now...
+                            trace!("GPU image not yet available; skipping batch for now.");
+                            None
+                        }
+                    } else {
+                        // Bind group already exists, meaning texture is ready
+                        Some(image_handle)
+                    }
+                } else {
+                    // Batch doesn't use particle texture
+                    None
+                };
 
-impl DrawEffects {
-    pub fn new(world: &mut World) -> Self {
-        Self {
-            params: SystemState::new(world),
+                // Specialize the render pipeline based on the effect batch
+                trace!(
+                    "Specializing render pipeline: shader={:?} particle_texture={:?}",
+                    batch.shader,
+                    particle_texture
+                );
+                let render_pipeline_id = specialized_render_pipelines.specialize(
+                    &mut render_pipeline_cache,
+                    &render_pipeline,
+                    ParticleRenderPipelineKey {
+                        particle_texture,
+                        shader: batch.shader.clone(),
+                        #[cfg(feature = "2d")]
+                        pipeline_mode: PipelineMode::Camera3d,
+                    },
+                );
+                trace!("Render pipeline specialized: id={:?}", render_pipeline_id);
+
+                // Add a draw pass for the effect batch
+                trace!("Add Transparent for batch on entity {:?}: buffer_index={} spawner_base={} slice={:?} handle={:?}", entity, batch.buffer_index, batch.spawner_base, batch.slice, batch.handle);
+                transparent_phase_3d.add(Transparent3d {
+                    draw_function: draw_effects_function_3d,
+                    pipeline: render_pipeline_id,
+                    entity,
+                    distance: 0.0, // TODO ??????
+                });
+            }
         }
     }
 }
@@ -1593,29 +1700,48 @@ pub struct ExtractedEffectEntities {
     pub entities: Vec<Entity>,
 }
 
-impl Draw<Transparent3d> for DrawEffects {
+/// Draw function for rendering all active effects for the current frame.
+///
+/// Effects are rendered in the [`Transparent2d`] phase of the main 2D pass,
+/// and the [`Transparent3d`] phase of the main 3D pass.
+pub struct DrawEffects {
+    params: SystemState<(
+        SRes<EffectsMeta>,
+        SRes<EffectBindGroups>,
+        SRes<PipelineCache>,
+        SQuery<Read<ViewUniformOffset>>,
+        SQuery<Read<EffectBatch>>,
+    )>,
+}
+
+impl DrawEffects {
+    pub fn new(world: &mut World) -> Self {
+        Self {
+            params: SystemState::new(world),
+        }
+    }
+}
+
+#[cfg(feature = "2d")]
+impl Draw<Transparent2d> for DrawEffects {
     fn draw<'w>(
         &mut self,
         world: &'w World,
         pass: &mut TrackedRenderPass<'w>,
         view: Entity,
-        item: &Transparent3d,
+        item: &Transparent2d,
     ) {
-        trace!("Draw<Transparent3d>: view={:?}", view);
-        let (
-            effects_meta,
-            image_bind_groups,
-            effect_bind_groups,
-            specialized_render_pipelines,
-            views,
-            effects,
-        ) = self.params.get(world);
+        trace!("Draw<Transparent2d>: view={:?}", view);
+        let (effects_meta, effect_bind_groups, specialized_render_pipelines, views, effects) =
+            self.params.get(world);
         let view_uniform = views.get(view).unwrap();
         let effects_meta = effects_meta.into_inner();
-        let image_bind_groups = image_bind_groups.into_inner();
         let effect_bind_groups = effect_bind_groups.into_inner();
         let effect_batch = effects.get(item.entity).unwrap();
-        if let Some(pipeline) = specialized_render_pipelines.into_inner().get(item.pipeline) {
+        if let Some(pipeline) = specialized_render_pipelines
+            .into_inner()
+            .get_render_pipeline(item.pipeline)
+        {
             trace!("render pass");
             //let effect_group = &effects_meta.effect_cache.buffers()[0]; // TODO
 
@@ -1647,7 +1773,85 @@ impl Draw<Transparent3d> for DrawEffects {
                 .contains(LayoutFlags::PARTICLE_TEXTURE)
             {
                 let image_handle = Handle::weak(effect_batch.image_handle_id);
-                if let Some(bind_group) = image_bind_groups.values.get(&image_handle) {
+                if let Some(bind_group) = effect_bind_groups.images.get(&image_handle) {
+                    pass.set_bind_group(2, bind_group, &[]);
+                } else {
+                    // Texture not ready; skip this drawing for now
+                    trace!(
+                        "Particle texture bind group not available for batch buf={} slice={:?}. Skipping draw call.",
+                        effect_batch.buffer_index,
+                        effect_batch.slice
+                    );
+                    return; //continue;
+                }
+            }
+
+            let vertex_count = effects_meta.vertices.len() as u32;
+            let particle_count = effect_batch.slice.end - effect_batch.slice.start;
+
+            trace!(
+                "Draw {} particles with {} vertices per particle for batch from buffer #{}.",
+                particle_count,
+                vertex_count,
+                effect_batch.buffer_index
+            );
+            pass.draw(0..vertex_count, 0..particle_count);
+        }
+    }
+}
+
+#[cfg(feature = "3d")]
+impl Draw<Transparent3d> for DrawEffects {
+    fn draw<'w>(
+        &mut self,
+        world: &'w World,
+        pass: &mut TrackedRenderPass<'w>,
+        view: Entity,
+        item: &Transparent3d,
+    ) {
+        trace!("Draw<Transparent3d>: view={:?}", view);
+        let (effects_meta, effect_bind_groups, specialized_render_pipelines, views, effects) =
+            self.params.get(world);
+        let view_uniform = views.get(view).unwrap();
+        let effects_meta = effects_meta.into_inner();
+        let effect_bind_groups = effect_bind_groups.into_inner();
+        let effect_batch = effects.get(item.entity).unwrap();
+        if let Some(pipeline) = specialized_render_pipelines
+            .into_inner()
+            .get_render_pipeline(item.pipeline)
+        {
+            trace!("render pass");
+            //let effect_group = &effects_meta.effect_cache.buffers()[0]; // TODO
+
+            pass.set_render_pipeline(pipeline);
+
+            // Vertex buffer containing the particle model to draw. Generally a quad.
+            pass.set_vertex_buffer(0, effects_meta.vertices.buffer().unwrap().slice(..));
+
+            // View properties (camera matrix, etc.)
+            pass.set_bind_group(
+                0,
+                effects_meta.view_bind_group.as_ref().unwrap(),
+                &[view_uniform.offset],
+            );
+
+            // Particles buffer
+            pass.set_bind_group(
+                1,
+                effect_bind_groups
+                    .render_particle_buffers
+                    .get(&effect_batch.buffer_index)
+                    .unwrap(),
+                &[],
+            );
+
+            // Particle texture
+            if effect_batch
+                .layout_flags
+                .contains(LayoutFlags::PARTICLE_TEXTURE)
+            {
+                let image_handle = Handle::weak(effect_batch.image_handle_id);
+                if let Some(bind_group) = effect_bind_groups.images.get(&image_handle) {
                     pass.set_bind_group(2, bind_group, &[]);
                 } else {
                     // Texture not ready; skip this drawing for now
@@ -1798,7 +2002,7 @@ impl Node for ParticleUpdateNode {
 
                         // Setup compute pass
                         //compute_pass.set_pipeline(&effect_group.compute_pipeline);
-                        compute_pass.set_pipeline(&compute_pipeline);
+                        compute_pass.set_pipeline(compute_pipeline);
                         compute_pass.set_bind_group(
                             0,
                             effects_meta.sim_params_bind_group.as_ref().unwrap(),
