@@ -6,18 +6,11 @@ use bevy::{
 };
 use std::hash::Hash;
 
+#[derive(Default)]
 pub struct PipelineRegistry {
     cache: HashMap<String, Handle<Shader>>,
 }
 
-impl Default for PipelineRegistry {
-    fn default() -> Self {
-        Self {
-            cache: Default::default(),
-        }
-    }
-}
-
 impl PipelineRegistry {
     pub fn configure(&mut self, source: &str, shaders: &mut Assets<Shader>) -> Handle<Shader> {
         if let Some(handle) = self.cache.get(source) {
